-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local im  = ui_imgui

local C = {}

C.name = 'Initialize Racer Vehicle'
C.description = 'Initialize Racer Vehicle.'
C.category = 'once_instant'
C.color = im.ImVec4(0.4, 0.9, 1.0, 0.9)
C.author = 'mathkuro'
C.tmpSecondPassFlag = true
C.pinSchema = {
  {dir = 'in', type = 'table', name = 'racerIdTable', description = 'Racer ID table.'},
  {dir = 'in', type = 'table', name = 'pathData', tableType = 'pathData',  description = 'Data from the path for other nodes to process.'},
  {dir = 'in', type = 'bool', name = 'reverse', hidden=true, default = false, hardcoded=true, description = 'Use the default reverse position instead.'},
  {dir = 'in', type = 'bool', name = 'lowPrecision', hidden=true, default = false, hardcoded=true, description = 'Use Low Prio if you want to move the vehicle in the first frame of it being spawned.'},
  {dir = 'in', type = 'bool', name = 'repair', hidden=true, default = true, hardcoded=true, description = 'If enabled, vehicle will be repaired. otherwise it will only be moved.'}
}

C.tags = {}

function C:moveToStartposition(path, startPositionName, vehId, lowPrecision, repair)
  local sp = path:findStartPositionByName(startPositionName)
  if sp == nil then
    log("W", "initializeRacerVehicleNode", "Could not find starting position!")
    return
  end

  print("Move to start position")
  sp:moveResetVehicleTo(vehId, lowPrecision or false, repair)
end

function C:workOnce()
  local racerInfoTable = {}
  for i, vid in ipairs(self.pinIn.racerIdTable.value) do
    local startPositionName = 'Racer Start '..i
    if self.pinIn.reverse.value then
      startPositionName = 'Racer Start '..i..' Reverse'
    end

    self:moveToStartposition(self.pinIn.pathData.value, startPositionName, vid, self.pinIn.lowPrecision.value or false, self.pinIn.repair.value)

    local veh = be:getObjectByID(vid)
    if veh then
      veh:queueLuaCommand("controller.getControllerSafe('esc').setESCMode(2)")
      veh:queueLuaCommand("electrics.values.underglow = 1")
      veh:queueLuaCommand("electrics.set_lightbar_signal(1)")
    end

    local startPosition = {pos=vec3(veh:getPositionXYZ()), dirFront=veh:getDirectionVector(), dirUp=veh:getDirectionVectorUp()}
    mathkuro_extraRaceMission.initializeRacerInfo(vid, startPositionName, startPosition)
  end
end

return _flowgraph_createNode(C)
